unit MFToolBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,stdctrls,buttons;

type
 TPanelState = (psOpen,psClosed);
 
type
 TTitleBar = class(TCustomPanel)
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
    procedure MouseMove(Shift: TShiftState;X,Y : Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
end;

type
 TTitle = class(TLabel)
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
    procedure MouseMove(Shift: TShiftState;X,Y : Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
end;


type
  TToolBox = class(TCustomPanel)
  private
   TitleBar : TTitleBar;
   Title : TTitle;
   ExpandBtn,HideBtn : TSpeedButton;
   FShrinkable,FClosable,Dragging : Boolean;
   UserDefinedHeight,XDown,YDown,XOff,YOff : Integer;
   FTitleBarCaption : string;
   FPanelState : TPanelState;
   function  GetTitle : string;
   procedure SetTitle(name : string);
   function  GetTitleBarColor : TColor;
   procedure SetTitleBarColor(Color : TColor);
   function  GetColor : TColor;
   procedure SetColor(theColor : TColor);
   function  GetPanelState : TPanelState;
   procedure SetPanelState(State : TPanelState);
   procedure DoHide(Sender : TObject);
   procedure DoChangeState(Sender : TObject);
   function  GetShrinkable : Boolean;
   procedure SetShrinkable(value : Boolean);
   function  GetClosable : Boolean;
   procedure SetClosable(value : Boolean);
  protected
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                       X,Y : Integer); override;
   procedure MouseMove(Shift: TShiftState;X,Y : Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
                     X,Y : Integer); override;
   procedure Resize; override;
  public
   constructor Create(AOwner : TComponent); override;
  published
    Property OnResize;
    property OnClick;
    Property OnDblClick;
    Property OnMouseDown;
    property OnMouseMove;
    Property OnMouseUp;
    Property OnEnter;
    Property OnExit;
    Property Enabled;
    Property Visible;
    Property ParentFont;
    Property Font;
    Property TitleBarText : string read GetTitle write SetTitle;
    Property TitleBarColor : TColor read GetTitleBarColor write SetTitleBarColor;
    Property ClientColor : TColor read GetColor write SetColor;
    Property PanelState : TPanelState read GetPanelState write SetPanelState;
    Property UserShrinkable : Boolean read GetShrinkable write SetShrinkable;
    Property UserClosable : Boolean read GetClosable write SetClosable;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MicroFrame', [TToolBox]);
end;

// ---- Title Bar Code -------------------------------------------------

procedure TTitleBar.MouseDown(Button: TMouseButton; Shift: TShiftState;
                              X,Y : Integer);
begin
 (Parent as TToolBox).MouseDown(Button,Shift,X,Y);
end;

procedure TTitleBar.MouseMove(Shift: TShiftState;X,Y : Integer);
begin
 (Parent as TToolBox).MouseMove(Shift,X,Y);
end;

procedure TTitleBar.MouseUp(Button: TMouseButton; Shift: TShiftState;
                            X,Y : Integer);
begin
 (Parent as TToolBox).MouseUp(Button,Shift,X,Y);
end;


// ---- Title Code -------------------------------------------------

procedure TTitle.MouseDown(Button: TMouseButton; Shift: TShiftState;
                              X,Y : Integer);
begin
 (Parent as TTitleBar).MouseDown(Button,Shift,X,Y);
end;

procedure TTitle.MouseMove(Shift: TShiftState;X,Y : Integer);
begin
 (Parent as TTitleBar).MouseMove(Shift,X,Y);
end;

procedure TTitle.MouseUp(Button: TMouseButton; Shift: TShiftState;
                            X,Y : Integer);
begin
 (Parent as TTitleBar).MouseUp(Button,Shift,X,Y);
end;


//------- Main Component Code ----------------------------------------------

constructor TToolBox.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 150;
 Height := 100;
 UserDefinedHeight := Height;
 FTitleBarCaption := 'Title';
 FPanelState := psOpen;
 TitleBar := TTitleBar.Create(self);
 TitleBar.Parent := self;
 TitleBar.Align := alTop;
 TitleBar.BevelOuter := bvLowered;
 TitleBar.Height := 23;
 TitleBar.Color := $00A00000;
 Title := TTitle.Create(self);
 Title.Parent := TitleBar;
 Title.Top := 4;
 Title.Left := 5;
 Title.ParentFont := false;
 Title.Font.Color := clWhite;
 Title.Font.Style := [fsBold];
 Title.Font.Size := -11;
 Title.Caption := FTitleBarCaption;
 HideBtn := TSpeedButton.Create(self);
 HideBtn.Parent := TitleBar;
 HideBtn.Width := 15;
 HideBtn.Height := 15;
 HideBtn.Top := 4;
 HideBtn.Left := Width - 22;
 HideBtn.ParentFont := false;
 HideBtn.Font.Style := [fsBold];
 HideBtn.Layout := blGlyphBottom;
 HideBtn.Caption := 'X';
 HideBtn.OnClick := DoHide;
 ExpandBtn := TSpeedButton.Create(self);
 ExpandBtn.Parent := TitleBar;
 ExpandBtn.Width := 15;
 ExpandBtn.Height := 15;
 ExpandBtn.Top := 4;
 ExpandBtn.Left := Width - 38;
 ExpandBtn.ParentFont := false;
 ExpandBtn.Font.Style := [fsBold];
 ExpandBtn.Layout := blGlyphBottom;
 ExpandBtn.Caption := '--';
 ExpandBtn.OnClick := DoChangeState;
end;

procedure TToolBox.MouseDown(Button: TMouseButton; Shift: TShiftState;
                               X,Y : Integer);
begin
 if Button = mbLeft then begin
  XDown := X;
  YDown := Y;
  FullRepaint := false;
  Dragging := true;
 end;
 inherited MouseDown(Button,Shift,X,Y);
end;

procedure TToolBox.MouseMove(Shift: TShiftState;X,Y : Integer);
begin
 if Dragging then begin
  XOff := X - XDown;
  YOff := Y - YDown;
  Left := Left + XOff;
  Top := Top + YOff;
 end;
 inherited MouseMove(Shift,X,Y);
end;

procedure TToolBox.MouseUp(Button: TMouseButton; Shift: TShiftState;
                             X,Y : Integer);
begin
 if (Dragging and (Button = mbLeft)) then Dragging := false;
 FullRepaint := true;
 inherited MouseUp(Button,Shift,X,Y);
end;

procedure TToolBox.DoHide(Sender : TObject);
begin
 Visible := false;
end;

procedure TToolBox.DoChangeState(Sender : TObject);
begin
 if FPanelState = psOpen then begin
  SetPanelState(psClosed);
  exit;
 end;
 if FPanelState = psClosed then SetPanelState(psOpen);
end;

procedure TToolBox.Resize;
begin
 HideBtn.Left := Width - 22;
 if HideBtn.Visible then ExpandBtn.Left := Width - 38;
 if not HideBtn.Visible then ExpandBtn.Left := Width - 22;
 if Width < 150 then Width := 150;
 if Height < 27 then Height := 27;
 inherited Resize;
end;


//------- Property Control Code --------------------------------------------


function TToolBox.GetTitle : string;
begin
 Result := FTitleBarCaption;
end;

procedure TToolBox.SetTitle(name : string);
begin
 if FTitleBarCaption <> name then begin
  FTitleBarCaption := name;
  Title.Caption := name;
 end;
end;

function TToolBox.GetTitleBarColor : TColor;
begin
 Result := TitleBar.Color;
end;

procedure TToolBox.SetTitleBarColor(Color : TColor);
begin
 if TitleBar.Color <> color then TitleBar.Color := color;
end;

function TToolBox.GetColor : TColor;
begin
 Result := Color;
end;

procedure TToolBox.SetColor(theColor : TColor);
begin
 if Color <> thecolor then Color := thecolor;
end;

function  TToolBox.GetPanelState : TPanelState;
begin
 Result := FPanelState;
end;

procedure TToolBox.SetPanelState(State : TPanelState);
begin
 if FPanelState = State then exit;
 if State = psClosed then begin
  UserDefinedHeight := Height;
  Height := 26;
  FPanelState := psClosed;
 end;
 if State = psOpen then begin
  Height := UserDefinedHeight;
  FPanelState := psOpen;
 end;
end;

function TToolBox.GetShrinkable : Boolean;
begin
 FShrinkable := ExpandBtn.Visible;
 Result := FShrinkable;
end;

procedure TToolBox.SetShrinkable(value : Boolean);
begin
 ExpandBtn.Visible := value;
 FShrinkable := value;
end;

function TToolBox.GetClosable : Boolean;
begin
 FClosable := HideBtn.Visible;
 Result := FClosable;
end;

procedure TToolBox.SetClosable(value : Boolean);
begin
 HideBtn.Visible := value;
 FClosable := value;
 if HideBtn.Visible then ExpandBtn.Left := Width - 38;
 if csDesigning in ComponentState then exit;
 if not HideBtn.Visible then ExpandBtn.Left := Width - 22;
end;


end.
